@@ -1,11 +1,18 @@
 #!/usr/bin/env python3
 """Process includes and variable substitutions in a YAML file.
 
-For each value `{"INCLUDE": "filename.yaml"}`, load content from include file
+For each value ``{"INCLUDE": "filename.yaml"}``, load content from include file
 and substitute the value with the content of the include file.
 
-For each string value with `$NAME` or `${NAME}` syntax, substitute with value
-of corresponding (environment) variable.
+For each string value with ``$NAME`` or ``${NAME}`` syntax, substitute with
+value of corresponding (environment) variable.
+
+For each string value with ``$YP_TIME_*`` or ``${YP_TIME_*}`` syntax,
+substitute with value of corresponding date-time string.
+
+Validate against specified JSON schema if root file starts with a
+``#!<SCHEMA-URI>`` line.
+
 """
 
 from argparse import ArgumentParser, RawDescriptionHelpFormatter
@@ -21,9 +28,11 @@ from urllib.parse import urlparse
 
 from dateutil.parser import parse as datetimeparse
 from dateutil.relativedelta import relativedelta
+from dateutil.tz import tzlocal
 import jmespath
 import jsonschema
 import yaml
+from yaml.dumper import SafeDumper
 
 from . import __version__
 
@@ -33,6 +42,13 @@ def configure_basic_logging():
 
     Basic no-frill format.
     Stream handler prints message on STDERR.
+
+    Normal usage:
+
+    >>> from yamlprocessor.dataprocess import DataProcessor
+    >>> processor = DataProcessor()
+    >>> # ... Customise the `DataProcessor` instance as necessary ..., then:
+    >>> processor.process_data(in_file_name, out_file_name)
     """
     logging.config.dictConfig({
         'version': 1,
@@ -66,6 +82,43 @@ def configure_basic_logging():
     })
 
 
+def strftime_with_colon_z(dto: datetime, time_format: str):
+    """Wrap dto.strftime to support %:z, %::z and %:::z format code.
+
+    Always use Z for UTC - it is short and recognised by any parser.
+    """
+    utcoffset = dto.utcoffset()
+    if utcoffset is None:
+        return dto.strftime(time_format)
+    # Hopefully, we don't need to have sub-seconds in time zones.
+    offset_total_seconds = int(utcoffset.total_seconds())
+    # Always use Z for UTC
+    if offset_total_seconds == 0:
+        for code in ('%z', '%:z', '%::z', '%:::z'):
+            time_format = time_format.replace(code, 'Z')
+        return dto.strftime(time_format)
+    # datetime.strftime can handle '%z' but not '%:z' etc
+    if not any(code in time_format for code in ('%:z', '%::z', '%:::z')):
+        return dto.strftime(time_format)
+    if offset_total_seconds >= 0:
+        offset_sign = '+'
+    else:
+        offset_sign = '-'
+    offset_total_seconds = abs(offset_total_seconds)
+    offset_str = '%s%02d:%02d:%02d' % (
+        offset_sign,
+        offset_total_seconds / 3600,  # hours
+        offset_total_seconds // 60 % 60,  # minutes of hour
+        offset_total_seconds % 60)  # seconds of minute
+    short_offset_str = offset_str
+    while short_offset_str.endswith(':00'):
+        short_offset_str = short_offset_str[0:-3]
+    time_format = time_format.replace('%:z', offset_str[0:6])
+    time_format = time_format.replace('%::z', offset_str)
+    time_format = time_format.replace('%:::z', short_offset_str)
+    return dto.strftime(time_format)
+
+
 class UnboundVariableError(ValueError):
 
     """An error raised on attempt to substitute an unbound variable."""
@@ -76,31 +129,87 @@ class UnboundVariableError(ValueError):
     __str__ = __repr__
 
 
+class YpSafeDumper(SafeDumper):
+    """Override dumping method for a time stamp to dump out str."""
+
+    @classmethod
+    def set_time_format(cls, time_format: str):
+        """Set time format."""
+        cls.time_format = time_format
+
+    def represent_datetime(self, data: datetime):
+        """Dump datetime as string."""
+        if hasattr(self, 'time_format'):
+            value = strftime_with_colon_z(data, self.time_format)
+            return self.represent_scalar('tag:yaml.org,2002:str', value)
+        else:
+            return super().represent_datetime(data)
+
+
+YpSafeDumper.add_representer(datetime, YpSafeDumper.represent_datetime)
+
+
 class DataProcessor:
     """Process YAML and compatible data structure.
 
     Import sub-data-structure from include files.
     Process variable substitution in string values.
+    Process date-time substitution in string values.
+    Validate against JSON schema.
 
-    Attributes:
-      `.is_process_include`:
-        (bool) Turn on/off include file processing.
-      `.is_process_variable`:
-        (bool) Turn on/off variable substitution.
-      `.include_paths`:
-        (list) Locations for searching include files.
-      `.schema_prefix`:
-        (str) Prefix for JSON schema specified as non-existing relative paths.
-      `.time_formats`:
-        (dict) Default and named time formats. (Default=`{'': '%FT%T%z'}`)
-      `.time_now`:
-        (datetime) Date-time at instance initialisation.
-      `.time_ref`:
-        (datetime) Reference date-time. (Default=`.time_now`)
-      `.variable_map`:
-        (dict) Mapping for variable substitutions. (Default=`os.environ`)
-      `.unbound_placeholder`:
-        (str) Value to substitute for unbound variables.
+    .. py:attribute:: .is_process_include
+       :type: bool
+
+       Turn on/off include file processing.
+
+    .. py:attribute:: .is_process_variable
+       :type: bool
+
+       Turn on/off variable substitution.
+
+    .. py:attribute:: .include_paths
+       :type: list
+
+       Locations for searching include files. Default is the value of the
+       :envvar:`YP_INCLUDE_PATH` environment variable split into a list.
+
+    .. py:attribute:: .schema_prefix
+       :type: str
+       :value: os.getenv("YP_SCHEMA_PREFIX")
+
+       Prefix for JSON schema specified as non-existing relative paths.
+       See also :envvar:`YP_SCHEMA_PREFIX`.
+
+    .. py:attribute:: .time_formats
+       :type: dict
+       :value: {'': '%FT%T%z'}
+
+       Default and named time formats.
+       See also :envvar:`YP_TIME_FORMAT` and :envvar:`YP_TIME_FORMAT_<NAME>`.
+
+    .. py:attribute:: .time_now
+       :type: datetime.datetime
+
+       Date-time at instance initialisation.
+
+    .. py:attribute:: .time_ref
+       :type: datetime.datetime
+
+       Reference date-time. Default is the value of the
+       :envvar:`YP_SCHEMA_PREFIX` environment variable as
+       :py:class:`datetime.datetime` or :py:attr:`.time_now` if the environment
+       variable is not defined.
+
+    .. py:attribute:: .variable_map
+       :type: dict
+       :value: os.environ
+
+       Mapping for variable substitutions.
+
+    .. py:attribute:: .unbound_placeholder
+       :type: str
+
+       Value to substitute for unbound variables.
     """
 
     INCLUDE_KEY = 'INCLUDE'
@@ -134,11 +243,18 @@ class DataProcessor:
     def __init__(self):
         self.is_process_include = True
         self.is_process_variable = True
-        self.include_paths = []
-        self.schema_prefix = None
-        self.time_formats = {'': '%FT%T%z'}
-        self.time_now = datetime.now()  # assume this application is fast
-        self.time_ref = self.time_now
+        self.include_paths = list(
+            item
+            for item in os.getenv('YP_INCLUDE_PATH', '').split(os.pathsep)
+            if item)
+        self.schema_prefix = os.getenv('YP_SCHEMA_PREFIX')
+        self.time_formats = {'': '%FT%T%:z'}
+        self.time_now = datetime.now(tzlocal())  # assume application is fast
+        time_ref_value = os.getenv('YP_TIME_REF_VALUE')
+        if time_ref_value is None:
+            self.time_ref = self.time_now
+        else:
+            self.time_ref = datetimeparse(time_ref_value)
         self.variable_map = os.environ.copy()
         self.unbound_placeholder = None
 
@@ -175,14 +291,20 @@ class DataProcessor:
             out_file = sys.stdout
         else:
             out_file = open(out_filename, 'w')
+        YpSafeDumper.set_time_format(self.time_formats[''])
         # Set sort_keys=False to preserve dict ordering (with Python 3.7+)
-        yaml.dump(root, out_file, default_flow_style=False, sort_keys=False)
+        yaml.dump(
+            root,
+            out_file,
+            Dumper=YpSafeDumper,
+            default_flow_style=False,
+            sort_keys=False)
         self.validate_data(root, out_filename, schema_location)
 
     def get_filename(self, filename: str, parent_filenames: list) -> str:
         """Return absolute path of filename.
 
-        If `filename` is a relative path, look for the file but looking the
+        If `filename` is a relative path, look for the file but looking in the
         directories containing the parent files, then the current working
         directory, then each path in `.include_paths`.
 
@@ -339,12 +461,12 @@ class DataProcessor:
             raise UnboundVariableError(name)
         for delta in deltas:
             dto = dto + delta
-        time_format_name = ''
+        time_fmt_key = ''
         match = self.REC_SUBSTITUTE_TIME_FORMAT.search(tail)
         if match:
-            time_format_name = match.groups()[0]
+            time_fmt_key = match.groups()[0]
         try:
-            return dto.strftime(self.time_formats[time_format_name])
+            return strftime_with_colon_z(dto, self.time_formats[time_fmt_key])
         except KeyError:
             raise UnboundVariableError(name)
 
@@ -359,8 +481,8 @@ class DataProcessor:
 
         NNN is in the same format as XXX, but the values are absolute
         information to replace the original date-time value. E.g. If the
-        current/reference time is 2022-01-14T12:00Z, applying _AT_1DT2H will
-        change the time to 2022-01-01T02:00Z.
+        current/reference time is 2022-01-14T12:30Z, applying _AT_1DT2H will
+        change the time to 2022-01-01T02:30Z.
 
         Deltas will be applied in the order they are defined.
 
@@ -455,7 +577,7 @@ def main(argv=None):
     parser.add_argument(
         '--include', '-I',
         dest='include_paths',
-        metavar='PATH',
+        metavar='DIR',
         action='append',
         default=[],
         help='Add search locations for item specified as relative paths')
@@ -499,7 +621,7 @@ def main(argv=None):
         '--schema-prefix',
         metavar='PREFIX',
         dest='schema_prefix',
-        default=os.getenv('YP_SCHEMA_PREFIX'),
+        default=None,
         help='Prefix for relative path schemas. (Override $YP_SCHEMA_PREFIX)')
     parser.add_argument(
         '--time-format',
@@ -516,7 +638,7 @@ def main(argv=None):
         '--time-ref',
         metavar='TIME',
         dest='time_ref',
-        default=os.getenv('YP_TIME_REF_VALUE'),
+        default=None,
         help=(
             'Reference value for date-time substitutions.'
             ' (Override $YP_TIME_REF_VALUE)'
@@ -553,7 +675,7 @@ def main(argv=None):
         processor.variable_map[key] = value
     processor.unbound_placeholder = args.unbound_placeholder
     # Date-time substitution options
-    if args.time_ref:
+    if args.time_ref is not None:
         processor.time_ref = datetimeparse(args.time_ref)
     for key, value in os.environ.items():
         if key == 'YP_TIME_FORMAT':
@@ -565,12 +687,13 @@ def main(argv=None):
         processor.time_formats[name] = value
     for item in args.time_formats:
         if '=' in item:
-            time_format_name, time_format = item.split('=', 1)
+            name, time_format = item.split('=', 1)
         else:
             name, time_format = ('', item)
         processor.time_formats[name] = time_format
     # Schema validation options
-    processor.schema_prefix = args.schema_prefix
+    if args.schema_prefix is not None:
+        processor.schema_prefix = args.schema_prefix
 
     processor.process_data(args.in_filename, args.out_filename)
 
